package pep;
import java.io.*;
import java.util.*;
import static pep.Main.nBlocks; import static pep.Main.isReplace;
import static pep.Main.nSamples;

/**This class creates the randomized trial order for each participant.*/
public class DataSet{
 private BufferedReader br;
 private String trials[] = new String[1000]; //array of trials
 private String trialsB[] = new String[1000]; //second array of trials
 private int nTrials = 0; //trial counter
 private Random r = new Random(); //random number generator
 private String set[] = new String[1000];
 private int counter;

 /**This method uses a source file to read what trials occur in a block,
  * randomizes them, and saves the complete sequence of trials to an array.
  * @param source The name of the text file containing the task stimuli.
  * @param subj The participant number.*/
 DataSet(String source,int subj){
  try{
   //setup input reader
   br = new BufferedReader(new FileReader(source));
   counter = 0;

   //read each line in the file and save to the trials[] array
   for(boolean isFileEnd=false; !isFileEnd; nTrials++){
    trials[nTrials] = new String(br.readLine());
    //recording stops at "END_OF_FILE" marker
    if(trials[nTrials].equals("END_OF_FILE")){
     isFileEnd=true; //stops the looping
     if(Main.twoTasks) splitTrials();
     nTrials--; //erases the END_OF_FILE entry
     mixTrials(); //randomizes the trials
    }
    //record "END_OF_BLOCK" marker
    else if(trials[nTrials].equals("END_OF_BLOCK")){
     if(Main.twoTasks) nTrials /= 2;
     nTrials--; //erases the END_OF_BLOCK entry
     mixTrials(); //randomizes the trial set
     nTrials=0; //erases the END_OF_FILE entry
    }
   }
   //leave an "END_OF_FILE" marker and close processes
   set[counter] = "END_OF_FILE";
   br.close();
  }
  catch(Exception e){
   System.out.println("Error reading source");
  }
 }

 /**This method randomizes the set of trials sent to it with one of several
  * randomization routines.*/
 private void mixTrials(){
  if(isReplace){ //unblocked, with replacement
   for(int sample=0; sample<nSamples; sample++){
    int randNum = r.nextInt(nTrials);
    set[counter] = trials[randNum];
    counter++;
   }
  }
  else{ //blocked, without replacement
   for(int block=0; block<nBlocks; block++){
    for(int trial=0; trial<nTrials; trial++){
     int randNum = r.nextInt(nTrials-trial);
     set[counter] = trials[randNum];
     counter++;
     String temp = trials[randNum];
     trials[randNum] = trials[nTrials-trial-1];
     trials[nTrials-trial-1] = temp;
     if(Main.twoTasks){ //alternating tasks
      randNum = r.nextInt(nTrials-trial);
      set[counter] = trialsB[randNum];
      counter++;
      temp = trialsB[randNum];
      trialsB[randNum] = trialsB[nTrials-trial-1];
      trialsB[nTrials-trial-1] = temp;
     }
    }
   }
  }
 }

 /**This method retrieves the indexed trial.
  * @param index The trial number.
  * @return The trail string.*/
 public String get(int index){
  return set[index];
 }

 /**This method splits trials up into two alternating tasks, using a second
  * trial list.*/
 private void splitTrials(){
  nTrials = ((nTrials+1) / 2); //each list will have half the trials
  for(int i=0; i<nTrials; i++){
   trialsB[i] = trials[nTrials+i]; //second half of trials
  }
  nTrials++;
 }
}