package pep;

import static pep.Main.nResponses;

/**This Node subclass defines the operations of Episode nodes.*/
public class Episode extends Node{
 /**search strength*/ private double searchStrength;
 /**response time*/   private double rt;

 /**Creates a new Episode.
  * @param subject Subject number used for static variable array.*/
 public Episode(int subject){
  super(subject,nResponses);
  threshold = .02;
  searchStrength = 6;
 }

 /**Updates activation for each cycle and searches responses.*/
 @Override
 public void activate(){
  if(input>0 || activation>.0001){ //skip if nothing meaningful to modify
   update();
   if(activation>threshold) respSearch();
  }
 }

 /**Activates Response nodes and weakens connections.*/
 public void respSearch(){
  double temp = (activation-threshold)*searchStrength;
  for(int j=0; j<nResponses; j++){
   double output = temp*connections[j];
   Main.runs[subj].responses[j].retrieved += output;
   Response.totRetrieved[subj] += output;
   connections[j] *= (1-(temp*decayConnect));
  }
 }

 /**Connects Episode to activated Response nodes.*/
 public void connect(){
  for(int j=0; j<nResponses; j++){
   connections[j] *= (1-writeConnect);
   connections[j] += Main.runs[subj].responses[j].activation*writeConnect;
  }
 }

 /**Tags an Episode as an error.*/
 public void tagError(){
  searchStrength = .02;
 }

 /**Updates activation for the newest episode.*/
 public void focus(){
  input += .05;
  update();
 }

 /**Records response time.
  * @param respTime Response time.*/
 public void setRT(double respTime){
  rt = respTime;
 }

 /**Retrieves response time.
  * @param cyc Cycle number.
  * @param connect Connection strength.
  * @return The current prediction strength of node.*/
 public double getRT(int cyc,double connect){
  double out = 0;
  if(cyc>0) out = (1 - (range(((double)Math.pow(cyc-rt,2))/10000)))*connect;
  return out;
 }
}