package pep;

/**This class defines the operations of Input nodes.*/
public class Input extends Node{
 /**stimulus bias*/   private double bias;
 /**stimulus signal*/ private double signal;

 /**Creates a new Input node.
  * @param subject Subject number used for static variable array.*/
 public Input(int subject){
  super(subject,RunTime.nEpisodes);
  threshold = .3;
  bias = 0;
  signal = 0;
 }

 /**Defines the input values for the node.
  * @param bs Amount of bias for the stimulus.
  * @param sig Amount of signal for the stimulus.*/
 public void param(double bs,double sig){
  bias = (java.lang.Math.random() + java.lang.Math.random());
  bias += (java.lang.Math.random() + java.lang.Math.random());
  bias /= 4; //normally distributed
  bias *= bs;
  signal = sig;
 }

 /**Calls the superclass method and activates Identity nodes.
  * @param stimNum The stimulus number.
  * @param isConflict Whether semantic conflict is present.*/
 public void activate(int stimNum,boolean isConflict){
  setInput();
  activate();
  if(isConflict) actOut(stimNum);
  else if(isTarget) actOut(stimNum);
 }

 /**Updates bias and sets the input for cycle.*/
 private void setInput(){
  bias = (bias*.996) + (signal*java.lang.Math.random()*.004);
  input += (bias + java.lang.Math.random())/2;
 }

 /**Reduces signal and bias when stimulus removed from display.*/
 public void remove(){
  signal *= .98;
  bias *= .98;
 }

 /**Reduces signal and bias when stimulus is masked.*/
 public void mask(){
  signal = 0;
 }

 /**Shuts off signal and bias.*/
 public void shutOff(){
  signal = 0;
  bias = 0;
 }

 /**Sends activation to Identity node.
  * @param stimNum The stimulus number.*/
 public void actOut(int stimNum){
  if(activation>threshold){
   double output = (activation-threshold)*java.lang.Math.random()*2;
   if(isTarget) output*=3;
   output = range(output);
   Main.runs[subj].identities[stimNum].input += output;
   Identity.totInput[subj] += output;
  }
 }
}