/**Parallel Episodic Processing (PEP) Model
 * Version 1.3.0
 * James R. Schmidt, Universiteit Gent
 * Fonds Wetenschappelijk Onderzoek - Vlaanderen (FWO)*/

package pep;

import java.util.concurrent.Semaphore;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class Main{
 //----------VARIABLES------------------------------------------
 /**DataSet source*/
 static String source = "wGratton.txt";
 /**number of subjects*/              static final int nSubjects = 1000;
 /**random with replacement?*/        static boolean isReplace = false;
 /**number of blocks per subject*/    static int nBlocks = 8;
 /**number of no replacement trials*/ static int nSamples = 0;
 /**number of responses*/             static int nResponses = 4;
 /**number of targets*/               static int nTargets = 4;
 /**target bias*/                     static final double tBias = .9;
 /**target signal*/                   static final double tSignal = .9;
 /**number of distracters per set*/   static final int[] nDistracters = {4};
 /**distracter bias*/                 static final double[] dBias = {.9};
 /**distracter signal*/               static final double[] dSignal = {.9};
 /**Set 1 distracter conflict*/       static boolean isConflict = true;
 /**conditions per variable*/         static final int[] numPerVar = {2};
 /**number of concurrent threads*/    public static final int maxThreads = 3;
 /**two alternating tasks?*/          public static boolean twoTasks = true;
 /**target onset time*/               public static int tOnset = 167;
 /**target offset time*/              public static int tOffset = 299;
 /**distracter onset time*/           public static int dOnset = 1;
 /**distracter offset time*/          public static int dOffset = 134;
 /**distracter mask*/                 public static int dMask = 167;
 //-------------------------------------------------------------
 /**thread manager*/
 public static final Semaphore freeThread = new Semaphore(maxThreads);
 /**array of runs*/
 public static RunTime runs[] = new RunTime[nSubjects];

 /**For each participant, the main method calls RunTime (the guts of the
  * model), and then generates condition means with DataAnalysis and
  * creates a concatenated datafile.*/
 public static void main(String[] args) throws InterruptedException{
  for(int y=0; y<nSubjects; y++){
   freeThread.acquire();
   runs[y] = new RunTime(y);
   Thread t = new Thread(runs[y]);
   t.start();
   System.out.println(""+(y+1)+" of "+nSubjects+" subjects started...");
  }

  //block until all threads are available (i.e., no active threads)
  freeThread.acquire(maxThreads);
  System.out.println("Run complete! Processing data...");

  //analyse data and create concatenated raw dataset
  DataAnalysis.main(new String[0]);
  System.out.println("Data analysis complete...");
  Concat.main(new String[0]);
  System.out.println("Data files concatenated...");
 }
}