package pep;

import java.io.*;
import java.util.*;
import static pep.Main.nTargets;   import static pep.Main.nDistracters;
import static pep.Main.nResponses; import static pep.Main.isConflict;
import static pep.Main.tSignal;    import static pep.Main.dSignal;
import static pep.Main.tBias;      import static pep.Main.dBias;
import static pep.Main.tOnset;     import static pep.Main.tOffset;
import static pep.Main.dOnset;     import static pep.Main.dOffset;
import static pep.Main.dMask;

/**This class is guts of the model, where each run is performed.*/
public class RunTime implements Runnable {
 /**number of distracter sets*/ static int nSets = nDistracters.length;
 /**number of variables*/       static int nVariables = Main.numPerVar.length;

 /**number of episodes*/
 static int nEpisodes = 1000; //should be greater than number of trials
 /**episodic memory*/
 Episode[] memory = new Episode[nEpisodes];
 /**target Input nodes*/
 Input[] targets = new Input[nTargets];
 /**distracter Input nodes*/
 Input[][] distracters = new Input[nSets][];
 /**Identity nodes*/
 Identity[] identities = new Identity[nTargets];
 /**Response nodes*/
 Response[] responses = new Response[nResponses];

 /**episode counter*/      int eNumber;
 /**end of trial marker*/  boolean isTrialOver;
 /**subject number*/       int subj;
 /**condition number*/     int[] condition = new int[nVariables];
 /**target number*/        int target;
 /**distracter number*/    int[] distracter = new int[nSets];
 /**cycle number*/         int cycle;
 /**correct response*/     int correctResp;
 /**actual response*/      int response;
 /**accuracy*/             int accuracy;
 /**stimulus on screen?*/  boolean isStimuli = true;
 /**end of trials*/        private boolean stop;
 /**number generator*/     private Random r = new Random();
 /**adjustable signal strength*/
 private double[] dSigTrend = (double[])dSignal.clone();
 /**signal deflection*/    private double deflect = 0;

 /**Sets up the thread.
  * @param subj The subject number.*/
 @SuppressWarnings("empty-statement")
 public RunTime(int subject){
  subj=subject;
 }

 /**Defines the nodes of the model defined and runs the trials.*/
 public void run(){
  //sets up the nodes for the distracter sets
  for(int set=0; set<nSets; set++){
   distracters[set] = new Input[nDistracters[set]];
   for(int k=0; k<nDistracters[set]; k++){
    distracters[set][k] = new Input(subj);
    distracters[set][k].ignored(nSets);
   }
  }
  //sets up the rest of the nodes
  for(int i=0; i<nEpisodes; i++) memory[i] = new Episode(subj);
  for(int i=0; i<nTargets; i++){
   targets[i] = new Input(subj);
   targets[i].targeted();
   identities[i] = new Identity(subj);
  }
  for(int i=0; i<nResponses; i++) responses[i] = new Response(subj);
  //set episode and connection counters to 0
  eNumber = 0; Node.nConnections[subj] = eNumber;

  //---Open I/O Streams---
  StringTokenizer tokenizer = null;
  FileOutputStream out = null;
  PrintStream p = null;
  DataSet trials = new DataSet(Main.source,subj+1);
  try{
   String outFile = "out"+(subj+1)+".txt";
   out = new FileOutputStream(outFile);
   p = new PrintStream(out);
   tokenizer = new StringTokenizer(trials.get(0)); //<--Trial 1
  }
  catch(Exception e){
   System.out.println("Error setting up input and output streams.");
  }

  //---Trial Loop---
  stop = false;
  while(!stop){
   //split up trial info into variables
   correctResp = Integer.parseInt(tokenizer.nextToken());
   target = Integer.parseInt(tokenizer.nextToken());
   for(int ns=0; ns<nSets; ns++)
    distracter[ns] = Integer.parseInt(tokenizer.nextToken());
   for(int nc=0; nc<nVariables; nc++)
    condition[nc] = Integer.parseInt(tokenizer.nextToken());
   response = 999; accuracy = 999;

   //---Attentional Wandering---
   //calculate change in distracter attention
   // <--add asterix between slashes to lesion attentional wandering
   double rand = (r.nextDouble() - r.nextDouble());
   deflect = deflect*.95+rand*.05;
   for(int j=0; j<nSets; j++)
    dSigTrend[j] = (dSigTrend[j]*.9) + (dSignal[j]*.1) + deflect;//*/

   //---Define Inputs and Run Cycles---
   //start with null input
   for(int i=0; i<nTargets; i++) targets[i].shutOff();
   for(int j=0; j<nSets; j++)
    for(int i=0; i<nDistracters[j]; i++) distracters[j][i].shutOff();
   //prepare loop
   isTrialOver = false; isStimuli = true;
   //the presented nodes are activated with noise plus a signal
   for(cycle=1; isTrialOver==false; cycle++){
    if(tOnset==cycle){
     for(int i=0; i<nTargets; i++) targets[i].param(tBias,0);
     targets[target].param(tBias,tSignal);
    }
    else if(tOffset==cycle) targets[target].remove();
    //distracters
    if(dOnset==cycle){
     for(int j=0; j<nSets; j++){
      for(int i=0; i<nDistracters[j]; i++)
       distracters[j][i].param(dBias[j],0);
      distracters[j][distracter[j]].param(dBias[j],dSigTrend[j]);
     }
    }
    else if(dOffset==cycle){
     for(int j=0; j<nSets; j++){
      for(int i=0; i<nDistracters[j]; i++)
       distracters[j][i].remove();
     }
    }
    else if(dMask==cycle){
     for(int j=0; j<nSets; j++){
      for(int i=0; i<nDistracters[j]; i++)
       distracters[j][i].mask();
     }
    }
    //run a cycle and monitor for colour and response identification
    go();
    if(monitorTarg()) if(monitorResp()) isTrialOver=true;
    if((cycle-tOnset)>1999) isTrialOver=true;
   }

   //---Post-Trial Coding---
   //calculate accuracy
   if(correctResp==response) accuracy = 1;
   else{
    accuracy = 0;
    memory[eNumber].tagError();
   }
   //write trial info to file
   p.print(""+(subj+1)+"\t"+response+"\t"+target+"\t");
   for(int k=0; k<nSets; k++) p.print(distracter[k]+"\t");
   for(int m=0; m<nVariables; m++) p.print(condition[m]+"\t");
   p.println(correctResp+"\t"+(cycle-tOnset)+"\t"+accuracy);
   //record response time into memory
   memory[eNumber].setRT(cycle-tOnset);

   //---Run ITI---
   isStimuli = false;
   //null input cycles (inter-trial interval)
   for(int i=0; i<nTargets; i++) targets[i].shutOff();
   for(int j=0; j<nSets; j++)
    for(int i=0; i<nDistracters[j]; i++) distracters[j][i].shutOff();
   //300 cycle ITI
   for(int i=0; i<300; i++){
    go();
   }

   //increase episode and connection counters
   eNumber++; Node.nConnections[subj] = eNumber;
   if(eNumber>=nEpisodes) System.out.println("Error: Not enough Episodes.");

   //---Read Next Trial---
   try{
    String tempTrial = new String(trials.get(eNumber));
    if(tempTrial.equals("END_OF_FILE")) stop=true;
    else tokenizer = new StringTokenizer(tempTrial);
   }
   catch(Exception e2){
    System.out.println("No END_OF_FILE marker");
    stop=true;
   }
  }

  //---Exit Simulation---
  p.println("END_OF_FILE"); //tag end of file
  //close input and output streams
  try{
   p.close(); out.close();
  }
  catch(Exception e3){
   System.out.println("Error closing input and output streams.");
  }

  //release thread to freeThread semaphore to allow a new thread
  Main.freeThread.release();
 }


 //-------------------------------------------------------------------
 //------------------------------Methods------------------------------
 //-------------------------------------------------------------------

 /**Performs all the calculations for an individual cycle.*/
 private void go(){
  //---INPUTS---
  for(int j=0; j<nSets; j++){
   boolean isConf = isConflict;
   if(j>0) isConf=false; //only the first node set conflicts
   for(int i=0; i<nDistracters[j]; i++) distracters[j][i].activate(i,isConf);
  }
  for(int i=0; i<nTargets; i++) targets[i].activate(i,isConflict);
  //---EPISODES---
  int eNumMinus10 = eNumber - 10;
  int eLoop = 0;
  while(eLoop<eNumMinus10){
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
   memory[eLoop].activate(); eLoop++;
  }
  for(; eLoop<eNumber; eLoop++) memory[eLoop].activate();
  memory[eNumber].focus();
  if(isStimuli) memory[eNumber].connect();
  //---IDENTITIES---
  for(int i=0; i<nTargets; i++) identities[i].activate(i);
  Identity.clear(subj);
  //---RESPONSES---
  for(int i=0; i<nResponses; i++) responses[i].activate();
  Response.clear(subj);
 }

 /**Checks colour identification.
  *@return If target is present.*/
 private boolean monitorTarg(){
  boolean found = false;
  for(int i=0; i<nTargets; i++)
   if(targets[i].activation>targets[i].threshold) found = true;
  return found;
 }

 /**Checks for temporal expectancy and generates a temporal bias.
  *@return Temporal expectancy bias.*/
 private double monitorTemp(){
  double expect = 0;
  // <--Insert asterix between slashes to lesion temporal learning
  //Limit search on initial trials (to avoid runtime errors)
  int iLimit = 5; if(eNumber<5) iLimit=eNumber;
  //Search for temporal expectancies
  for(int i=1; i<=iLimit; i++){
   double str = Math.pow((6-i),3)*.002;
   expect += memory[eNumber-i].getRT(cycle-tOnset,str);
  }
  expect -= .05; //small threshold .05
  if(expect<0) expect=0;
  else if(expect>.25) expect=.25; //max expectancy drop
  //*/
  return expect;
 }

 /**Checks for response identification and ends the trial if necessary.
  *@return If the response has been found.*/
 private boolean monitorResp(){
  boolean found = false;
  for(int i=0; i<nResponses; i++){
   if(responses[i].activation>(responses[i].threshold-monitorTemp())){
    found = true;
    //if two responses cross threshold at same time, response = 777
    if(response != 999) response = 777;
    else response = i;
   }
  }
  return found;
 }
}